MODULE zdfevd_crs
   !!======================================================================
   !!                       ***  MODULE  zdfevd  ***
   !! Ocean physics: parameterization of convection through an enhancement
   !!                of vertical eddy mixing coefficient
   !!======================================================================
   !! History :  OPA  !  1997-06  (G. Madec, A. Lazar)  Original code
   !!   NEMO     1.0  !  2002-06  (G. Madec)  F90: Free form and module
   !!             -   !  2005-06  (C. Ethe) KPP parameterization
   !!            3.2  !  2009-03  (M. Leclair, G. Madec, R. Benshila) test on both before & after
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   zdf_evd      : increase the momentum and tracer Kz at the location of
   !!                  statically unstable portion of the water column (ln_zdfevd=T)
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers variables
   USE dom_oce         ! ocean space and time domain variables
   USE zdf_oce         ! ocean vertical physics variables
   USE zdfkpp          ! KPP vertical mixing
   USE in_out_manager  ! I/O manager
   USE iom             ! for iom_put
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE timing          ! Timing
   USE wrk_nemo       ! work arrays
   USE crs

   IMPLICIT NONE
   PRIVATE

   PUBLIC   zdf_evd_crs    ! called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011)
   !! $Id: zdfevd.F90 4990 2014-12-15 16:42:49Z timgraham $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE zdf_evd_crs( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE zdf_evd  ***
      !!                   
      !! ** Purpose :   Local increased the vertical eddy viscosity and diffu-
      !!      sivity coefficients when a static instability is encountered.
      !!
      !! ** Method  :   avt, avm, and the 4 neighbouring avmu, avmv coefficients
      !!      are set to avevd (namelist parameter) if the water column is 
      !!      statically unstable (i.e. if rn2 < -1.e-12 )
      !!
      !! ** Action  :   avt, avm, avmu, avmv updted in static instability cases
      !!
      !! References :   Lazar, A., these de l'universite Paris VI, France, 1997
      !!----------------------------------------------------------------------
      !USE oce,   zavt_evd => ua , zavm_evd => va  ! (ua,va) used ua workspace
      !
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step indexocean time step
      !
      INTEGER ::   ji, jj, jk   ! dummy loop indices
      REAL(wp), POINTER, DIMENSION(:,:,:) :: zavt_evd_crs,zavm_evd_crs
      !!----------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('zdf_evd_crs')
      !
      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'zdf_evd_crs : Enhanced Vertical Diffusion (evd)'
         IF(lwp) WRITE(numout,*) '~~~~~~~ '
         IF(lwp) WRITE(numout,*)
      ENDIF

      CALL wrk_alloc( jpi_crs,jpj_crs,jpk, zavt_evd_crs, zavm_evd_crs )

      zavt_evd_crs(:,:,:) = avt_crs(:,:,:)           ! set avt prior to evd application

      SELECT CASE ( nn_evdm )
      !
      !CASE ( 1 )           ! enhance vertical eddy viscosity and diffusivity (if rn2<-1.e-12)
      !   !
      !   zavm_evd(:,:,:) = avm_crs(:,:,:)           ! set avm prior to evd application
      !   !
      !   DO jk = 1, jpkm1 
      !      DO jj = 2, jpj_crs             ! no vector opt.
      !         DO ji = 2, jpi_crs
      !            IF(  MIN( rn2_crs(ji,jj,jk), rb2_crs(ji,jj,jk) ) <= -1.e-12 ) THEN
      !               avt_crs (ji  ,jj  ,jk) = rn_avevd * tmask_crs(ji  ,jj  ,jk)
      !               avm_crs (ji  ,jj  ,jk) = rn_avevd * tmask_crs(ji  ,jj  ,jk)
      !               avmu_crs(ji  ,jj  ,jk) = rn_avevd * umask_crs(ji  ,jj  ,jk)
      !               avmu_crs(ji-1,jj  ,jk) = rn_avevd * umask_crs(ji-1,jj  ,jk)
      !               avmv_crs(ji  ,jj  ,jk) = rn_avevd * vmask_crs(ji  ,jj  ,jk)
      !               avmv_crs(ji  ,jj-1,jk) = rn_avevd * vmask_crs(ji  ,jj-1,jk)
      !            ENDIF
      !         END DO
      !      END DO
      !   END DO 
      !   CALL crs_lbc_lnk( avt_crs , 'W', 1. )   ;   CALL crs_lbc_lnk( avm_crs , 'W', 1. )   ! Lateral boundary conditions
      !   CALL crs_lbc_lnk( avmu_crs, 'U', 1. )   ;   CALL crs_lbc_lnk( avmv_crs, 'V', 1. )
      !   !
      !   zavm_evd_crs(:,:,:) = avm_crs(:,:,:) - zavm_evd_crs(:,:,:)   ! change in avm due to evd
      !   CALL iom_put( "avm_evd", zavm_evd_crs )              ! output this change
      !   !
      CASE DEFAULT         ! enhance vertical eddy diffusivity only (if rn2<-1.e-12) 
         DO jk = 1, jpkm1
            DO jj = 1, jpj_crs             ! loop over the whole domain (no lbc_lnk call)
               DO ji = 1, jpi_crs
                  IF(  MIN( rn2_crs(ji,jj,jk), rb2_crs(ji,jj,jk) ) <= -1.e-12 )   &
                     avt_crs(ji,jj,jk) = rn_avevd * tmask_crs(ji,jj,jk)
               END DO
            END DO
         END DO
         !
      END SELECT 

      zavt_evd_crs(:,:,:) = avt_crs(:,:,:) - zavt_evd_crs(:,:,:)   ! change in avt due to evd
      CALL iom_put( "avt_evd_crs", zavt_evd_crs )              ! output this change
      !
      CALL wrk_dealloc( jpi_crs,jpj_crs,jpk, zavt_evd_crs, zavm_evd_crs )
      !
      IF( nn_timing == 1 )  CALL timing_stop('zdf_evd_crs')
      !
   END SUBROUTINE zdf_evd_crs

   !!======================================================================
END MODULE zdfevd_crs
